/*
Copyright © 2024 Mabel

All rights reserved. This plugin is protected by copyright law.

You may not modify, redistribute, or resell this software without explicit written permission from the copyright owner.

For any support please message me directly via Discord `mabel8686` or join my discord https://discord.gg/YWzEJVt89V

███╗   ███╗ █████╗ ██████╗ ███████╗██╗
████╗ ████║██╔══██╗██╔══██╗██╔════╝██║
██╔████╔██║███████║██████╔╝█████╗  ██║
██║╚██╔╝██║██╔══██║██╔══██╗██╔══╝  ██║
██║ ╚═╝ ██║██║  ██║██████╔╝███████╗███████╗
╚═╝     ╚═╝╚═╝  ╚═╝╚═════╝ ╚══════╝╚══════╝
*/
using Newtonsoft.Json;
using System;
using System.Collections.Generic;
using System.Linq;
using UnityEngine;

namespace Oxide.Plugins
{
    [Info("Drop Inventory", "Mabel", "1.0.2")]
    [Description("Drops a player's inventory on command")]
    class DropInventory : RustPlugin
    {
        private const string permission_use = "dropinventory.use";
        private const string permission_bypass = "dropinventory.cooldown.bypass";
        private const string permission_admin = "dropinventory.admin";

        private Dictionary<ulong, float> cooldowns = new Dictionary<ulong, float>();

        static Configuration config;

        public class BroadcastSettings
        {
            [JsonProperty("Chat Prefix")]
            public string chatPrefix { get; set; }

            [JsonProperty("Use Chat Message")]
            public bool useChat { get; set; }

            [JsonProperty("Use Game Tip Message")]
            public bool useTip { get; set; }
        }

        public class DropSettings
        {
            [JsonProperty("Sound Effect")]
            public string soundEffect { get; set; }

            [JsonProperty("Drop Velocity")]
            public float dropVelocity { get; set; }

            [JsonProperty("Drop Main Container")]
            public bool dropMain { get; set; }

            [JsonProperty("Drop Wear Container")]
            public bool dropWear { get; set; }

            [JsonProperty("Drop Belt Container")]
            public bool dropBelt { get; set; }

            [JsonProperty("Drop Items In Bag")]
            public bool dropInBag { get; set; }
        }

        public class CommandSettings
        {
            [JsonProperty("Chat Command Cooldown")]
            public float commandCooldown { get; set; }

            [JsonProperty("Player Chat Command")]
            public string playerCommand { get; set; }

            [JsonProperty("Admin Chat Command")]
            public string adminCommand { get; set; }

            [JsonProperty("Admin Console Command")]
            public string consoleCommand { get; set; }
        }

        public class Configuration
        {
            [JsonProperty(PropertyName = "Broadcast Settings")]
            public BroadcastSettings BroadcastSettings { get; set; }

            [JsonProperty(PropertyName = "Drop Inventory Settings")]
            public DropSettings DropSettings { get; set; }

            [JsonProperty(PropertyName = "Command Settings")]
            public CommandSettings CommandSettings { get; set; }

            public Core.VersionNumber Version { get; set; }

            public static Configuration DefaultConfig()
            {
                return new Configuration
                {
                    BroadcastSettings = new BroadcastSettings()
                    {
                        chatPrefix = "<color=#FF0000>[DropInv]</color> ",
                        useChat = false,
                        useTip = true,
                    },
                    DropSettings = new DropSettings()
                    {
                        soundEffect = "assets/bundled/prefabs/fx/gestures/drink_vomit.prefab",
                        dropVelocity = 5f,
                        dropMain = true,
                        dropWear = true,
                        dropBelt = true,
                    },
                    CommandSettings = new CommandSettings()
                    {
                        commandCooldown = 60f,
                        playerCommand = "drop",
                        adminCommand = "drop.admin",
                        consoleCommand = "drop.admin"
                    },
                    Version = new Core.VersionNumber()
                };
            }
        }

        protected override void LoadConfig()
        {
            base.LoadConfig();
            try
            {
                config = Config.ReadObject<Configuration>();
                if (config == null) LoadDefaultConfig();
                SaveConfig();

                if (config.Version < Version)
                    UpdateConfig();

                Config.WriteObject(config, true);
            }
            catch (Exception ex)
            {
                Debug.LogException(ex);
                PrintWarning("Creating new configuration file....");
                LoadDefaultConfig();
            }
        }

        private void UpdateConfig()
        {
            PrintWarning("Config update detected! Updating config values...");

            Configuration baseConfig = Configuration.DefaultConfig();
            if (config.Version < new Core.VersionNumber(1, 0, 0))
                config = baseConfig;

            config.Version = Version;
            PrintWarning("Config update completed!");
        }

        protected override void LoadDefaultConfig()
        {
            config = Configuration.DefaultConfig();
            PrintWarning("Default configuration has been loaded....");
        }
        protected override void SaveConfig()
        {
            Config.WriteObject(config);
        }

        protected override void LoadDefaultMessages()
        {
            lang.RegisterMessages(new Dictionary<string, string>
            {
                ["Cooldown"] = "Drop is on cooldown. Time remaining: <color=#FF0000>{0}</color> seconds.",
                ["DropAll"] = "Your Main, Belt and Wear items have been dropped.",
                ["DropMain"] = "Your Main items have been dropped.",
                ["DropBelt"] = "Your Belt items have been dropped.",
                ["DropWear"] = "Your Wear items have been dropped.",
                ["DropMainBelt"] = "Your Main and Belt items have been dropped.",
                ["DropMainWear"] = "Your Main and Wear items have been dropped.",
                ["DropBeltWear"] = "Your Belt and Wear items have been dropped.",
                ["NoContainers"] = "No containers are enabled for dropping items.",
                ["AdminDroppedItems"] = "Your items were dropped by an admin.",
                ["AdminDroppedItemsReply"] = "You dropped (Player: <color=#FF0000>{0}</color>) Items.",
                ["NoPermission"] = "You don't have permission to use this command.",
                ["AdminCommandUsage"] = "Usage: /drop.admin <playername or steamid>",
                ["PlayerNotFound"] = "Player not found or multiple players found matching the name or ID provided.",
                ["Restrained"] = "You Can Not Use This Command While Restrained"
            }, this);
        }

        void Init()
        {
            permission.RegisterPermission(permission_use, this);
            permission.RegisterPermission(permission_bypass, this);
            permission.RegisterPermission(permission_admin, this);
            cmd.AddChatCommand(config.CommandSettings.playerCommand, this, nameof(DropInventoryCommand));
            cmd.AddChatCommand(config.CommandSettings.adminCommand, this, nameof(DropInventoryAdminCommand));
            cmd.AddConsoleCommand(config.CommandSettings.consoleCommand, this, nameof(DropInventoryAdminConsoleCommand));
        }

        private Vector3 GetRandomVelocity()
        {
            return new Vector3(UnityEngine.Random.Range(-config.DropSettings.dropVelocity, config.DropSettings.dropVelocity), config.DropSettings.dropVelocity, UnityEngine.Random.Range(-config.DropSettings.dropVelocity, config.DropSettings.dropVelocity));
        }

        private bool CanUseCommand(BasePlayer player)
        {
            if (player.IsRestrained)
            {
                player?.ShowToast(GameTip.Styles.Red_Normal, lang.GetMessage("Restrained", this, player.UserIDString));
                return false;
            }

            if (permission.UserHasPermission(player.UserIDString, permission_bypass)) return true;

            ulong userId = player.userID;
            float currentTime = Time.realtimeSinceStartup;

            float lastUsageTime;
            if (cooldowns.TryGetValue(userId, out lastUsageTime))
            {
                float timeRemaining = lastUsageTime + config.CommandSettings.commandCooldown - currentTime;
                int roundedTimeRemaining = Mathf.CeilToInt(timeRemaining);

                if (timeRemaining > 0f)
                {
                    if (config.BroadcastSettings.useChat)
                    {
                        var message = $"{config.BroadcastSettings.chatPrefix} {string.Format(lang.GetMessage("Cooldown", this, player.UserIDString), roundedTimeRemaining)}";
                        SendReply(player, message);
                    }

                    if (config.BroadcastSettings.useTip)
                    {
                        player?.ShowToast(GameTip.Styles.Blue_Normal, string.Format(lang.GetMessage("Cooldown", this, player.UserIDString), roundedTimeRemaining));
                    }
                    return false;
                }
            }

            cooldowns[userId] = currentTime;
            return true;
        }

        void DropInventoryCommand(BasePlayer player, string command, string[] args)
        {
            if (!permission.UserHasPermission(player.UserIDString, permission_use))
            {
                if (config.BroadcastSettings.useChat)
                {
                    var message = $"{config.BroadcastSettings.chatPrefix} {string.Format(lang.GetMessage("NoPermission", this, player.UserIDString))}";
                    SendReply(player, message);
                }

                if (config.BroadcastSettings.useTip)
                {
                    player?.ShowToast(GameTip.Styles.Red_Normal, string.Format(lang.GetMessage("NoPermission", this, player.UserIDString)));
                }
                return;
            }

            if (!CanUseCommand(player))
                return;

            if (!string.IsNullOrEmpty(config.DropSettings.soundEffect))
            {
                Effect.server.Run(config.DropSettings.soundEffect, player.transform.position);
            }

            DropPlayerItems(player);

            if (config.BroadcastSettings.useChat)
            {
                var message = $"{config.BroadcastSettings.chatPrefix} {GetDropMessage(player)}";
                SendReply(player, message);
            }

            if (config.BroadcastSettings.useTip)
            {
                player?.ShowToast(GameTip.Styles.Blue_Normal, GetDropMessage(player));
            }
        }

        void DropInventoryAdminCommand(BasePlayer player, string command, string[] args)
        {
            if (!permission.UserHasPermission(player.UserIDString, permission_admin))
            {
                if (config.BroadcastSettings.useChat)
                {
                    var message = $"{config.BroadcastSettings.chatPrefix} {lang.GetMessage("NoPermission", this, player.UserIDString)}";
                    SendReply(player, message);
                }

                if (config.BroadcastSettings.useTip)
                {
                    player?.ShowToast(GameTip.Styles.Red_Normal, string.Format(lang.GetMessage("NoPermission", this, player.UserIDString)));
                }
                return;
            }

            if (args.Length < 2)
            {
                if (config.BroadcastSettings.useChat)
                {
                    var message = $"{config.BroadcastSettings.chatPrefix} {lang.GetMessage("AdminCommandUsage", this, player.UserIDString)}";
                    SendReply(player, message);
                }

                if (config.BroadcastSettings.useTip)
                {
                    player?.ShowToast(GameTip.Styles.Blue_Normal, string.Format(lang.GetMessage("AdminCommandUsage", this, player.UserIDString)));
                }
                return;
            }

            string dropType = args[0].ToLower();
            string targetPlayerName = args[1];

            BasePlayer targetPlayer = FindPlayer(targetPlayerName);
            if (targetPlayer == null)
            {
                if (config.BroadcastSettings.useChat)
                {
                    var message = $"{config.BroadcastSettings.chatPrefix} {lang.GetMessage("PlayerNotFound", this, player.UserIDString)}";
                    SendReply(player, message);
                }

                if (config.BroadcastSettings.useTip)
                {
                    player?.ShowToast(GameTip.Styles.Red_Normal, string.Format(lang.GetMessage("PlayerNotFound", this, player.UserIDString)));
                }
                return;
            }

            if (!CanUseCommand(targetPlayer))
                return;

            if (!string.IsNullOrEmpty(config.DropSettings.soundEffect))
            {
                Effect.server.Run(config.DropSettings.soundEffect, targetPlayer.transform.position);
            }

            switch (dropType)
            {
                case "all":
                    DropItemsFromContainer(targetPlayer.inventory?.containerMain, targetPlayer, true);
                    DropItemsFromContainer(targetPlayer.inventory?.containerBelt, targetPlayer, true);
                    DropItemsFromContainer(targetPlayer.inventory?.containerWear, targetPlayer, true);
                    break;
                case "belt":
                    DropItemsFromContainer(targetPlayer.inventory?.containerBelt, targetPlayer, true);
                    break;
                case "wear":
                    DropItemsFromContainer(targetPlayer.inventory?.containerWear, targetPlayer, true);
                    break;
                case "main":
                    DropItemsFromContainer(targetPlayer.inventory?.containerMain, targetPlayer, true);
                    break;
                default:
                    if (config.BroadcastSettings.useChat)
                    {
                        SendReply(player, $"{config.BroadcastSettings.chatPrefix} Invalid drop type. Valid types: all, belt, wear, main.");
                    }
                    return;
            }

            if (config.BroadcastSettings.useChat)
            {
                var message = $"{config.BroadcastSettings.chatPrefix} {lang.GetMessage("AdminDroppedItemsReply", this, player.UserIDString)} {targetPlayer.displayName}";
                SendReply(player, message);

                var message1 = $"{config.BroadcastSettings.chatPrefix} {lang.GetMessage("AdminDroppedItems", this, targetPlayer.UserIDString)}";
                SendReply(targetPlayer, message1);
            }

            if (config.BroadcastSettings.useTip)
            {
                player?.ShowToast(GameTip.Styles.Blue_Normal, string.Format(lang.GetMessage("AdminDroppedItemsReply", this, player.UserIDString), targetPlayer.displayName));
                targetPlayer?.ShowToast(GameTip.Styles.Blue_Normal, string.Format(lang.GetMessage("AdminDroppedItems", this, player.UserIDString)));
            }
        }

        void DropInventoryAdminConsoleCommand(ConsoleSystem.Arg arg)
        {
            if (arg?.Args == null || arg.Args.Length < 2)
            {
                SendReply(arg, $"Usage: {config.CommandSettings.adminCommand} (all|belt|wear|main) (playername or steamid)");
                return;
            }

            string dropType = arg.Args[0].ToLower();
            string targetPlayerName = arg.Args[1];

            BasePlayer targetPlayer = BasePlayer.Find(targetPlayerName);
            if (targetPlayer == null)
            {
                SendReply(arg, "Player not found or multiple players found matching the name or ID provided.");
                return;
            }

            switch (dropType)
            {
                case "all":
                    DropItemsFromContainer(targetPlayer.inventory?.containerMain, targetPlayer, true);
                    DropItemsFromContainer(targetPlayer.inventory?.containerBelt, targetPlayer, true);
                    DropItemsFromContainer(targetPlayer.inventory?.containerWear, targetPlayer, true);
                    break;
                case "belt":
                    DropItemsFromContainer(targetPlayer.inventory?.containerBelt, targetPlayer, true);
                    break;
                case "wear":
                    DropItemsFromContainer(targetPlayer.inventory?.containerWear, targetPlayer, true);
                    break;
                case "main":
                    DropItemsFromContainer(targetPlayer.inventory?.containerMain, targetPlayer, true);
                    break;
                default:
                    SendReply(arg, "Invalid drop type. Valid types: all, belt, wear, main.");
                    return;
            }

            SendReply(arg, $"You dropped {dropType} items from Player: {targetPlayer.displayName}'s inventory.");

            if (config.BroadcastSettings.useChat)
            {
                var message = $"{config.BroadcastSettings.chatPrefix} {lang.GetMessage("AdminDroppedItems", this, targetPlayer.UserIDString)}";
                SendReply(targetPlayer, message);
            }

            if (config.BroadcastSettings.useTip)
            {
                targetPlayer?.ShowToast(GameTip.Styles.Blue_Normal, string.Format(lang.GetMessage("AdminDroppedItems", this, targetPlayer.UserIDString)));
            }
        }

        private BasePlayer FindPlayer(string target)
        {
            if (ulong.TryParse(target, out ulong steamId))
            {
                return BasePlayer.FindByID(steamId);
            }

            var players = BasePlayer.activePlayerList
                .Where(p => p.displayName.Contains(target, StringComparison.OrdinalIgnoreCase));
            if (players.Count() == 1)
            {
                return players.First();
            }
            return null;
        }

        private string GetDropMessage(BasePlayer player)
        {
            bool mainContainerEnabled = config.DropSettings.dropMain;
            bool beltContainerEnabled = config.DropSettings.dropBelt;
            bool wearContainerEnabled = config.DropSettings.dropWear;

            if (!mainContainerEnabled && !beltContainerEnabled && !wearContainerEnabled)
                return lang.GetMessage("NoContainers", this, player.UserIDString);
            else if (mainContainerEnabled && beltContainerEnabled && wearContainerEnabled)
                return lang.GetMessage("DropAll", this, player.UserIDString);
            else if (mainContainerEnabled && beltContainerEnabled)
                return lang.GetMessage("DropMainBelt", this, player.UserIDString);
            else if (mainContainerEnabled && wearContainerEnabled)
                return lang.GetMessage("DropMainWear", this, player.UserIDString);
            else if (beltContainerEnabled && wearContainerEnabled)
                return lang.GetMessage("DropBeltWear", this, player.UserIDString);
            else if (mainContainerEnabled)
                return lang.GetMessage("DropMain", this, player.UserIDString);
            else if (beltContainerEnabled)
                return lang.GetMessage("DropBelt", this, player.UserIDString);
            else if (wearContainerEnabled)
                return lang.GetMessage("DropWear", this, player.UserIDString);

            return "Unknown drop combination.";
        }

        private void DropPlayerItems(BasePlayer player)
        {
            DropItemsFromContainer(player.inventory?.containerMain, player, config.DropSettings.dropMain);
            DropItemsFromContainer(player.inventory?.containerBelt, player, config.DropSettings.dropBelt);
            DropItemsFromContainer(player.inventory?.containerWear, player, config.DropSettings.dropWear);
        }

        private void DropItemsFromContainer(ItemContainer container, BasePlayer player, bool shouldDrop)
        {
            if (container == null || !shouldDrop) return;

            List<Item> itemsToDrop = new List<Item>(container.itemList);
            foreach (var item in itemsToDrop)
            {
                var randomVelocity = GetRandomVelocity();
                item.Drop(player.GetDropPosition(), randomVelocity);
            }
        }
    }
}